@@ -1,4 +1,4 @@
-# Copyright 2013 OpenStack, LLC
+# Copyright (c) 2013 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -119,6 +119,34 @@ def _stub_backup(id, base_uri, tenant_id):
     }
 
 
+def _stub_qos_full(id, base_uri, tenant_id, name=None, specs=None):
+    if not name:
+        name = 'fake-name'
+    if not specs:
+        specs = {}
+
+    return {
+        'qos_specs': {
+            'id': id,
+            'name': name,
+            'consumer': 'back-end',
+            'specs': specs,
+        },
+        'links': {
+            'href': _bookmark_href(base_uri, tenant_id, id),
+            'rel': 'bookmark'
+        }
+    }
+
+
+def _stub_qos_associates(id, name):
+    return {
+        'assoications_type': 'volume_type',
+        'name': name,
+        'id': id,
+    }
+
+
 def _stub_restore():
     return {'volume_id': '712f4980-5ac1-41e5-9383-390aa7c9f58b'}
 
@@ -251,8 +279,10 @@ class FakeHTTPClient(base_client.HTTPClient):
         action = body.keys()[0]
         if action == 'os-reset_status':
             assert 'status' in body['os-reset_status']
+        elif action == 'os-update_snapshot_status':
+            assert 'status' in body['os-update_snapshot_status']
         else:
-            raise AssertionError('Unexpected action: %s" % action')
+            raise AssertionError('Unexpected action: %s' % action)
         return (resp, {}, _body)
 
     #
@@ -283,6 +313,10 @@ class FakeHTTPClient(base_client.HTTPClient):
         r = {'volume': self.get_volumes_detail()[2]['volumes'][0]}
         return (200, {}, r)
 
+    def get_volumes_1234_encryption(self, **kw):
+        r = {'encryption_key_id': 'id'}
+        return (200, {}, r)
+
     def post_volumes_1234_action(self, body, **kw):
         _body = None
         resp = 202
@@ -309,6 +343,9 @@ class FakeHTTPClient(base_client.HTTPClient):
             assert 'status' in body[action]
         elif action == 'os-extend':
             assert body[action].keys() == ['new_size']
+        elif action == 'os-migrate_volume':
+            assert 'host' in body[action]
+            assert 'force_host_copy' in body[action]
         else:
             raise AssertionError("Unexpected action: %s" % action)
         return (resp, {}, _body)
@@ -390,6 +427,11 @@ class FakeHTTPClient(base_client.HTTPClient):
                           'name': 'test-type-1',
                           'extra_specs': {}}})
 
+    def get_types_2(self, **kw):
+        return (200, {}, {'volume_type': {'id': 2,
+                          'name': 'test-type-2',
+                          'extra_specs': {}}})
+
     def post_types(self, body, **kw):
         return (202, {}, {'volume_type': {'id': 3,
                           'name': 'test-type-3',
@@ -405,6 +447,23 @@ class FakeHTTPClient(base_client.HTTPClient):
     def delete_types_1(self, **kw):
         return (202, {}, None)
 
+    #
+    # VolumeEncryptionTypes
+    #
+    def get_types_1_encryption(self, **kw):
+        return (200, {}, {'id': 1, 'volume_type_id': 1, 'provider': 'test',
+                          'cipher': 'test', 'key_size': 1,
+                          'control_location': 'front'})
+
+    def get_types_2_encryption(self, **kw):
+        return (200, {}, {})
+
+    def post_types_2_encryption(self, body, **kw):
+        return (200, {}, {'encryption': {}})
+
+    def put_types_1_encryption_1(self, body, **kw):
+        return (200, {}, {})
+
     #
     # Set/Unset metadata
     #
@@ -481,6 +540,69 @@ class FakeHTTPClient(base_client.HTTPClient):
         return (200, {},
                 {'restore': _stub_restore()})
 
+    #
+    # QoSSpecs
+    #
+
+    def get_qos_specs_1B6B6A04_A927_4AEB_810B_B7BAAD49F57C(self, **kw):
+        base_uri = 'http://localhost:8776'
+        tenant_id = '0fa851f6668144cf9cd8c8419c1646c1'
+        qos_id1 = '1B6B6A04-A927-4AEB-810B-B7BAAD49F57C'
+        return (200, {},
+                _stub_qos_full(qos_id1, base_uri, tenant_id))
+
+    def get_qos_specs(self, **kw):
+        base_uri = 'http://localhost:8776'
+        tenant_id = '0fa851f6668144cf9cd8c8419c1646c1'
+        qos_id1 = '1B6B6A04-A927-4AEB-810B-B7BAAD49F57C'
+        qos_id2 = '0FD8DD14-A396-4E55-9573-1FE59042E95B'
+        return (200, {},
+                {'qos_specs': [
+                    _stub_qos_full(qos_id1, base_uri, tenant_id, 'name-1'),
+                    _stub_qos_full(qos_id2, base_uri, tenant_id)]})
+
+    def post_qos_specs(self, **kw):
+        base_uri = 'http://localhost:8776'
+        tenant_id = '0fa851f6668144cf9cd8c8419c1646c1'
+        qos_id = '1B6B6A04-A927-4AEB-810B-B7BAAD49F57C'
+        qos_name = 'qos-name'
+        return (202, {},
+                _stub_qos_full(qos_id, base_uri, tenant_id, qos_name))
+
+    def put_qos_specs_1B6B6A04_A927_4AEB_810B_B7BAAD49F57C(self, **kw):
+        return (202, {}, None)
+
+    def put_qos_specs_1B6B6A04_A927_4AEB_810B_B7BAAD49F57C_delete_keys(
+            self, **kw):
+        return (202, {}, None)
+
+    def delete_qos_specs_1B6B6A04_A927_4AEB_810B_B7BAAD49F57C(self, **kw):
+        return (202, {}, None)
+
+    def get_qos_specs_1B6B6A04_A927_4AEB_810B_B7BAAD49F57C_associations(
+            self, **kw):
+        type_id1 = '4230B13A-7A37-4E84-B777-EFBA6FCEE4FF'
+        type_id2 = '4230B13A-AB37-4E84-B777-EFBA6FCEE4FF'
+        type_name1 = 'type1'
+        type_name2 = 'type2'
+        return (202, {},
+                {'qos_associations': [
+                    _stub_qos_associates(type_id1, type_name1),
+                    _stub_qos_associates(type_id2, type_name2)]})
+
+    def get_qos_specs_1B6B6A04_A927_4AEB_810B_B7BAAD49F57C_associate(
+            self, **kw):
+        return (202, {}, None)
+
+    def get_qos_specs_1B6B6A04_A927_4AEB_810B_B7BAAD49F57C_disassociate(
+            self, **kw):
+        return (202, {}, None)
+
+    def get_qos_specs_1B6B6A04_A927_4AEB_810B_B7BAAD49F57C_disassociate_all(
+            self, **kw):
+        return (202, {}, None)
+
+    #
     #
     # VolumeTransfers
     #
