@@ -114,6 +114,15 @@ class Volume(base.Resource):
 
         self.manager.extend(self, volume, new_size)
 
+    def migrate_volume(self, host, force_host_copy):
+        """Migrate the volume to a new host."""
+        self.manager.migrate_volume(self, host, force_host_copy)
+
+#    def migrate_volume_completion(self, old_volume, new_volume, error):
+#        """Complete the migration of the volume."""
+#        self.manager.migrate_volume_completion(self, old_volume,
+#                                               new_volume, error)
+
 
 class VolumeManager(base.ManagerWithFind):
     """
@@ -134,13 +143,13 @@ class VolumeManager(base.ManagerWithFind):
         :param display_name: Name of the volume
         :param display_description: Description of the volume
         :param volume_type: Type of volume
-        :rtype: :class:`Volume`
         :param user_id: User id derived from context
         :param project_id: Project id derived from context
         :param availability_zone: Availability Zone to use
         :param metadata: Optional metadata to set on volume creation
         :param imageRef: reference to an image stored in glance
         :param source_volid: ID of source volume to clone from
+        :rtype: :class:`Volume`
         """
 
         if metadata is None:
@@ -352,3 +361,37 @@ class VolumeManager(base.ManagerWithFind):
         return self._action('os-extend',
                             base.getid(volume),
                             {'new_size': new_size})
+
+    def get_encryption_metadata(self, volume_id):
+        """
+        Retrieve the encryption metadata from the desired volume.
+
+        :param volume_id: the id of the volume to query
+        :return: a dictionary of volume encryption metadata
+        """
+        return self._get("/volumes/%s/encryption" % volume_id)._info
+
+    def migrate_volume(self, volume, host, force_host_copy):
+        """Migrate volume to new host.
+
+        :param volume: The :class:`Volume` to migrate
+        :param host: The destination host
+        :param force_host_copy: Skip driver optimizations
+        """
+
+        return self._action('os-migrate_volume',
+                            volume,
+                            {'host': host, 'force_host_copy': force_host_copy})
+
+    def migrate_volume_completion(self, old_volume, new_volume, error):
+        """Complete the migration from the old volume to the temp new one.
+
+        :param old_volume: The original :class:`Volume` in the migration
+        :param new_volume: The new temporary :class:`Volume` in the migration
+        :param error: Inform of an error to cause migration cleanup
+        """
+
+        new_volume_id = base.getid(new_volume)
+        return self._action('os-migrate_volume_completion',
+                            old_volume,
+                            {'new_volume': new_volume_id, 'error': error})[1]
